write_serializable_bytes_without_length

Function write_serializable_bytes_without_length 

pub fn write_serializable_bytes_without_length<T, A, F>(
    elements: &[A],
    output: &mut [u8],
)
Writes the bytes for a VarZeroLengthlessSlice into an output buffer. Usable for a VarZeroSlice if you first write the length bytes.

Every byte in the buffer will be initialized after calling this function.

Panics

Panics if the buffer is not exactly the correct length.